# -*- coding: utf-8 -*-
# tatorter
# Copyright (C) 2018  Daniel Llin Ferrero
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''
Created on 05.02.2018

@author: DLF
'''
from fuzzywuzzy import fuzz
import os

class Episode(object):
    def __init__(self, **kwargs):
        self._attribs = []
        for key, val in kwargs.items():
            setattr(self, key, val)
            self._attribs.append(key)
        for must_have in ["episode_index","title","location","broadcaster"]:
            assert must_have in self._attribs
    @property
    def as_dict(self):
        result = {}
        for attrib in self._attribs:
            result[attrib] = getattr(self, attrib)
        return result
    def match(self, file_name):
        file_name = os.path.splitext(os.path.basename(file_name))[0].lower()
        title = self.title.lower()
        broadcaster = self.broadcaster.lower()
        value = fuzz.partial_ratio(title, file_name)
        #max 100
        if broadcaster in file_name:
            value +=10
        #max 110
        if str(self.episode_index) in file_name:
            value += 5 * len(str(self.episode_index))
        #max 130
        value = int(100 * value / 130)
        return value  
        
    
class Matcher(object):
    def __init__(self, file_name, episodes):
        self.match_list = []
        for episode in episodes:
            value = episode.match(file_name)
            self.match_list.append((value, episode))
        self.match_list.sort(key=lambda e:e[0], reverse=True)
    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# tatorter
# Copyright (C) 2018  Daniel Llin Ferrero
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''
Created on 05.02.2018

@author: DLF
'''
from os.path import expanduser
from datetime import datetime
from pathlib import Path
from shutil import move
import argparse
import logging
import json
import glob
import sys
import os
from tatorterp import Episode, Matcher
from tatorterp.grabber import WikipdediaDEGrabber

logger = logging.getLogger("tatorter")
logging.basicConfig()
logger.setLevel(logging.INFO)

file_rename_pattern = "{episode_index:0>4}--{location}--{title}--({team})"

def start():
    # get arguments
    home = expanduser("~")
    default_cache_path = "{}/.tatorter.cache".format(home)
    argparser = argparse.ArgumentParser()
    argparser.add_argument(
        "-c","--cache",
        help="file for caching episode data (default: {})".format(default_cache_path),
        default=default_cache_path
        )
    argparser.add_argument(
        "-r","--renew-cache",
        help="force the cache file to be renewed, even it is not out-dated".format(default_cache_path),
        action='store_true'
        )
    argparser.add_argument(
        "files",
        nargs='*'
        )
    args = argparser.parse_args()
    
    # options, not configurable
    cache_days = 1
    
    cache_path = Path(args.cache)
    cache_used = False
    episodes = None
    
    # handle cache, load data (from cache or Wikipdeia)
    if args.renew_cache:
        logger.info("Forcing cache update.")
    else:
        if cache_path.is_file():
            last_modified_date = datetime.fromtimestamp(os.path.getmtime(cache_path))
            now = datetime.now()
            if (now - last_modified_date).days < cache_days:
                cache_used = True
                logging.info("Loading cache...")
                with open(cache_path,mode="r",encoding="utf-8") as cache_file:
                    cache = json.load(cache_file)
                    episodes = []
                    for e in cache:
                        episodes.append(Episode(**e))
            else:
                logger.info("Cache out-dated. ({} days)".format((now - last_modified_date).days))
        else:
            logger.info("No cache file.")
    if not cache_used:
        logging.info("Fetching online data...")
        episodes = WikipdediaDEGrabber().episodes
        logger.info("Storing cache...")
        with open(cache_path,mode="w",encoding="utf-8") as cache_file:
            json.dump([episode.as_dict for episode in episodes], cache_file)
    
    # identify files to process
    files = []
    for f in args.files:
        files.extend(glob.glob(f))
    
    # handle each file separately
    options_count = 5
    for file in files:
        matcher = Matcher(file, episodes)
        source_file = os.path.basename(file)
        target_path = os.path.dirname(file)
        target_extension = os.path.splitext(file)[1]
        target_names = []
        print ("="*80)
        print ("Choose new name for \"{}\"".format(source_file)) 
        for ix in range(options_count):
            match = matcher.match_list[ix]
            target_name = file_rename_pattern.format(**(match[1].as_dict)) + target_extension
            target_name = target_name.replace("/","⁄")
            target_names.append(target_name)
            print ("–"*80 + "\n" + "{ix:>5} {match:>6}| {target_name}".format(
                ix=ix+1,
                match="({}%)".format(match[0]),
                target_name = target_name
                ))
        selection = None
        while selection is None:
            print ("–"*80)
            selection_text = input("Choose a new file name ([1..{max}]), skip this file (s) or cancel this script (q):".format(max=options_count))
            if selection_text in [str(i+1) for i in range(options_count)]:
                selection = int(selection_text)
            if selection_text in ['s', 'q']:
                selection = selection_text
        if selection == 's':
            continue
        if selection == 'q':
            sys.exit(0)
        full_target_name = os.path.join(target_path, target_names[selection-1])
        if Path(full_target_name).is_file():
            selection_text = input("Target file {} exists! Overwrite? (y/n)".format(full_target_name))
            while selection_text.lower() not in ['y','n']:
                selection_text = input("I just understand 'y' or 'n'. Overwrite? (y/n)".format(full_target_name))
            if selection_text == 'n':
                continue
        try:
            move(file, full_target_name)
            print ("Moved to {}".format(full_target_name))
        except IOError as e:
            print("File could not be copied ({0}): {1}".format(e.errno, e.strerror))   
        print ("")


if __name__ == "__main__":
    start()
    
    


# -*- coding: utf-8 -*-
# tatorter
# Copyright (C) 2018  Daniel Llin Ferrero
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''
Created on 04.02.2018

@author: DLF
'''

import mechanicalsoup as ms
import logging
from tatorterp import Episode
logger = logging.getLogger("tatorter")
logging.basicConfig()

team_to_location = {
    "Faber, Bönisch und Dalay":"Dortmund",
    "Sieland, Gorniak und Schnabel":"Dresden",
    "Ballauf und Schenk":"Köln",
    "Eisner und Fellner":"Wien",
    "Odenthal und Kopper":"Ludwigshafen",
    "Stellbrink und Marx":"Saarbrücken",
    "Lessing und Dorn":"Weimar",
    "Falke und Grosz":"Hamburg",
    "Thiel und Boerne":"Münster",
    "Lindholm":"Hannover",
    "Janneke und Brix":"Frankfurt",
    "Lürsen und Stedefreund":"Bremen",
    "Lannert und Bootz":"Stuttgart",
    "Batic und Leitmayr":"München",
    "Tobler und Berg":"Freiburg",
    "Flückiger und Ritschard":"Luzern",
    "Borowski und Brandt":"Kiel",
    "Rubin und Karow":"Berlin",
    "Faber, Bönisch, Dalay und Kossik":"Dortmund",
    "Voss, Ringelhahn, Goldwasser, Fleischer und Schatz":"Franken",
    "Blum und Perlmann, Matteo Lüthi":"Konstanz",
    "Blum und Perlmann":"Konstanz",
    "Saalfeld und Keppler":"Leipzig",
    "Flückiger und Lanning":"Luzern",
    "Eisner":"Wien",
    "Ritter und Stark":"Berlin",
    "Steier und Mey":"Frankfurt",
    "Murot":"Wiesbaden",
    "Batu":"Hamburg",
    "Kappl und Deininger":"Saarbrücken",
    "Saalfeld und Keppler /Ballauf und Schenk":"Leipzig-Köln",
    "Ballauf und Schenk /Saalfeld und Keppler":"Köln-Leipzig",
    "Tschiller und Gümer":"Hamburg",
    "Falke und Lorenz":"Hamburg",
    "Funck, Schaffert und Grewel":"Erfurt",
    "Steier":"Frankfurt",
    "Sieland, Gorniak, Mohr und Schnabel":"Dresden",
    "Berlinger":"Freiburg",
    "Borowski /Lindholm":"Kiel-Hannover",
    "Stark":"Berlin",
    "Schimanski und Thanner":"Duisburg"
    }

class WikipdediaDEGrabber(object):
    url = "https://de.wikipedia.org/wiki/Liste_der_Tatort-Folgen"
    def __init__(self):
        browser = ms.StatefulBrowser()
        browser.open(WikipdediaDEGrabber.url)
        tables = browser.get_current_page().find_all(name='table', attrs={'class':'wikitable sortable'})
        assert len(tables) == 2, "Page content unexpected, not exactly two wikitables. Cannot parse it."
        tbodys = tables[0].find_all("tbody")
        assert len(tbodys) == 1, "Page content unexpected, more than one tbody in wikitable. Cannot parse it."
        trs = tbodys[0].find_all("tr")
        logger.info("Parsing {} epsiode rows from {}.".format(len(trs), WikipdediaDEGrabber.url))
        self.episodes = []
        last_epsiode = None
        last_values = None
        for tr in trs[1:]:
            tds = tr.find_all('td')
            values = [td.text.split('(')[0].strip() for td in tds]
            episode_index = int(values[0].replace('a*','').replace('b*',''))
            # some special treatments
            if episode_index == 835:
                assert len(values) == 7
                values[6] = last_epsiode.author
                values.append(last_epsiode.director)
            elif episode_index in [854, 970]:
                assert len(values) == 5
                last_values[1] = values[1]
                last_values[3] = values[2]
                last_values[5] = values[3]
                values = last_values
            team = values[4]
            if team not in team_to_location:
                if episode_index > 800:
                    logger.warning("Location for team {} unknown and episode later than 800. Script should be updated!".format(team))
                location = "[{}]".format(team)
            else:
                location = team_to_location[team]
            episode = Episode(
                episode_index = episode_index,
                location = location,
                title = values[1],
                broadcaster = values[2],
                premiere = values[3],
                team = team,
                case_index = values[5],
                author = values[6],
                director = values[7]
            )
            self.episodes.append(episode)
            last_epsiode = episode
            last_values = values
            
                
        
        
#print (WikipdediaDEGrabber().episodes)

    


# tatorter
# Copyright (C) 2018  Daniel Llin Ferrero
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''
Created on 27.11.2017

:author: DLF
'''

from setuptools import setup
from setuptools import find_packages


setup(
    name='tatorter',
    version='0.1',
    description='A simple script to rename Tatort episode video files to a unique pattern.',
    author='DLFW',
    package_dir={'': 'src'},
    packages=find_packages('src'),
    install_requires=['fuzzywuzzy','python-Levenshtein','mechanicalsoup','pathlib'],
    license='GPL 3',
    scripts=['scripts/tatorter','scripts/tatorter.py'],
    classifiers=[
        'Environment :: Console',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
    ],
)
